using namespace std;
/*************************************************************** 
* 
* DalDataComponent.cc & XMM Science Analysis System (SAS) 
* 
* Class representing DAL DataComponent.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalDataComponent.cc,v 1.2 2004/03/05 14:43:01 eojero Exp $
* 
*****************************************************************/ 


#include <errstr.h>
#include <Tracer.h>
#include "DalDataComponent.h"


static DalNullable::NullType
toDataType( Dal::DataType dataType )
{
	PROC

	DalNullable::NullType	toType = DalNullable::Undefined;

	switch( dataType )
	{
		case Dal::Int8 :
		case Dal::Uint16 :
		case Dal::Int16 :
		case Dal::Int32 :
		case Dal::Uint32 :
			toType = DalNullable::Integer;
			break;
		case Dal::Real32 :
		case Dal::Real64 :
			toType = DalNullable::Real;
			break;
		case Dal::DString :
			toType = DalNullable::String;
			break;
		case Dal::Bool :
			toType = DalNullable::Undefined;
			break;
		default :
			errstr << "Invalid switch option: " << int( dataType ) << Sas::error( "internalError" );
			break;
	} // switch

	RETURN toType;
}


DalDataComponent::DalDataComponent(
	DataType			dataType,
	const longVec&			size,
	const string &			units,
	const Labelled*			labelled )
	:
		DalNullable( toDataType( dataType )),
		_labelled( labelled ),
		_dataType( dataType ),
		_size( size ),
		_units( units ),
		_scaled( false ),
		_scaleZero( 0 ),
		_scaleFactor( 1 )
{
	PROC
	ENDPROC
}


DalDataComponent::~DalDataComponent()
{
	PROC

	if( _dataObjects.size() )
	{
		errstr << "An unreleased data object has been detected: " << Sas::error( "unreleasedObject" );

	} // if
	ENDPROC
}


void
DalDataComponent::checkUnreleasedData()
{
	PROC

	while( _dataObjects.size() )
	{
		// ********************************************************************************
		// We are assuming that the Attributable base class
		// has not been destroyed.
		// For example, we are assumming that class Column is defined as
		// class Column : ... public virtual Attributable, ... public virtual DataComponent
		// When class Column has the above defintion, the class
		// DataComponent will be destroyed before class Attributable.
		// This allows us to safely use the member _labelled.
		// ********************************************************************************
		//errstr << "An unreleased data object has been detected: The object to which this data relates has qualified name '" << _labelled -> qualifiedName() << "'" << Sas::warning( "unreleasedData" );
		errstr << "An unreleased data object has been detected: The object to which this data relates has qualified name '" << _labelled -> qualifiedName() << "'" << Sas::message( Msg::Core, Msg::Sparse );

		delete _dataObjects[ _dataObjects.size() - 1 ];
	} // while
	ENDPROC
}


//DalDataComponent::DalDataComponent(
//	const DataComponent&		c )
//	:	DalNullable( c ),
//		_dataType( c.dataType() ),
//		_size( c.size() ),
//		_units( c.units() ),
//		_scaled( false ),
//		_scaleZero( 0 ),
//		_scaleFactor( 1 )
//{
//	PROC
//	ENDPROC
//}


DalDataComponent&
DalDataComponent::operator=( const Column& c )
{
	PROC
	(DalNullable&)*this = c;
	_size = c.size();
	_units = c.units();
	_scaled = c.scaled();
	_scaleZero = c.scaleZero();
	_scaleFactor = c.scaleFactor();
	_labelled = &c;
	RETURN *this;
}


//DalDataComponent&
//DalDataComponent::operator=( const DataComponent& c )
//{
//	PROC
//	(DalNullable&)*this = c;
//	_size = c.size();
//	_units = c.units();
//	_scaled = c.scaled();
//	_scaleZero = c.scaleZero();
//	_scaleFactor = c.scaleFactor();
//	RETURN *this;
//}


const string&
DalDataComponent::units() const
{
	PROC
	RETURN _units;
}


void
DalDataComponent::units(const string& units)
{
	PROC
	_units = units;
	ENDPROC
}


void
DalDataComponent::scaling(
		double			zero,
		double			scale )
{
	PROC

	_scaleZero = zero;
	_scaleFactor = scale;
	_scaled = true;

	ENDPROC
}


void
DalDataComponent::scale( bool onoff )
{
	PROC
	_scaled = onoff;
	ENDPROC
}


bool
DalDataComponent::scaled() const
{
	PROC
	RETURN _scaled;
}

		
double
DalDataComponent::scaleFactor() const
{
	PROC RETURN _scaleFactor;
}


double
DalDataComponent::scaleZero() const
{
	PROC
	RETURN _scaleZero;
}


Dal::DataType
DalDataComponent::dataType() const
{
	PROC
	RETURN _dataType;
}


const longVec&
DalDataComponent::size() const
{
	PROC
	RETURN _size;
}


unsigned int
DalDataComponent::dimensions() const
{
	PROC
	RETURN _size.size();
}


void
DalDataComponent::removeDataObject( Data* thisDataObject )
{
	PROC

	bool found = false;

	// ***********************************************
	// Do the search from the end of the list. This
	// ensures that the object is found quickly
	// in the event that the objects are being deleted
	// in reverse order.
	// ***********************************************
	for(
		vector<Data*>::iterator i = _dataObjects.end() - 1;
		i >= _dataObjects.begin();
		--i )
	{
		Data* d = (*i);
		if( d == thisDataObject )
		{
			found = true;
			_dataObjects.erase( i );
			break;
		} // if
	} // for

	if( !found )
	{
		errstr << "No such column data object" << Sas::error( "invalidDataObject" );
	} // if
	ENDPROC
}


void
DalDataComponent::addDataObject( Data* thisDataObject )
{
	PROC
	_dataObjects.push_back( thisDataObject );
	ENDPROC
}


void
DalDataComponent::release()
{
	PROC

	while( _dataObjects.size() )
	{
		delete _dataObjects[ _dataObjects.size() - 1 ];
		//_dataObjects.erase( _dataObjects.end() - 1 );
	} // while
	ENDPROC
}



