using namespace std;
/*************************************************************** 
* 
* HiLowColumnData.cc           & XMM Science Analysis System (SAS) 
* 
* Class representing the Data in a Column. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowColumnData.cc,v 1.2 2004/03/05 14:43:02 eojero Exp $
* 
*****************************************************************/ 


#include "errstr.h"
#include "Tracer.h"
#include "vectorutil.h"
#include "HiLowColumnData.h"
#include "Storage.h"

#include <cassert>

HiLowColumnData::HiLowColumnData(
			HiLowDataStorage *		data,
			const DalColumn *		parent,
			unsigned long			from,
			unsigned long			count,
			AccessMode 			accessMode )

		:	DalColumnData( parent, from, count ),
			HiLowData<DalColumn>( data, 0, 0, parent, accessMode )
{
	PROC

	seek( from, count );
	//dataStorage() -> initialise( seekable().from(), seekable().count() );

	ENDPROC
}


HiLowColumnData::~HiLowColumnData()
{
	PROC

	// ************************************************
	// Ensure that this ColumnData is the parent of the
	// DataStorage object.
	// ************************************************
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );

	DalColumn * column = this -> DalChild<DalColumn>::parent();
	dataStorage() -> release(
				from() * column -> elements(),
				count() * column -> elements() );

	column -> removeDataObject( this );

	ENDPROC
}


unsigned long
HiLowColumnData::position(
		unsigned long			row ) const
{
	PROC

	if ( elements() == 0 )
		Sas::error("noVarSizeColumn","var-size cells not supported");

	assert( row >= from() && row < from() + count() );

	RETURN (row - from()) * elements();
}

unsigned long
HiLowColumnData::position(
		unsigned long			row,
		unsigned long			pos) const
{
	PROC

	if ( elements() == 0 )
		Sas::error("noVarSizeColumn","var-size cells not supported");

	assert( row >= from() && row < from() + count() && pos < elements() );

	RETURN ( row - from() ) * elements() + pos;

}


unsigned long
HiLowColumnData::position(
		unsigned long			row,
		const longVec& 			pos) const
{
	PROC

	const Column * column = this -> DalChild<Column>::parent();

	if ( elements() == 0 )
		Sas::error("noVarSizeColumn","var-size cells not supported");

	if ( row < from() || row >= from() + count() )
	{
 		errstr << "cannot access row " << row << " since the data ranges from " << from() <<" to " << from()+count() << Sas::error("invalidRow");
	} // if

	unsigned dim = size().size();

	if ( pos.size() != dim )
	{
		errstr << "the position has dimension " << pos.size() << " whereas the column " << column->name() << " has dimension " << dim << Sas::error("invalidPosition");
	} // if

	unsigned long factor = 1;

	unsigned long idx = 0;

	for (unsigned i = dim; i; --i)
	{
		unsigned ix = i-1;
		if ( pos[ix] >= size()[ix] )
		{
			errstr << "cannot access position" << pos << " since the cell data has size" << size() << Sas::error("invalidPosition");

		} // if

		idx += factor * pos[ix];
		factor *= size()[ix];
	} // for

	idx += (row - from()) * factor;

	RETURN idx;
}



void
HiLowColumnData::seek( unsigned long from, unsigned long count )
{
	PROC

	// Release existing, data associated with this
	// ColumnData, but it might be necessary
	// not to do this on construction i.e. [0,0] case
	//dataStorage() -> release( seekable().from(), seekable().count() );

	unsigned long	_from;
	unsigned long	_count;
	Column *	column = this -> DalChild<Column>::parent();

	if( count == 0 )
	{
		if( column -> count() == 0 )
		{
			_from = 0;
			_count = column -> rows();
		}
		else
		{
			_from = column -> from();
			_count = column -> count();
		} // if
	}
	else
	{
		_from = from;
		_count = count;
	} // if

	DalSeekable< unsigned long >::seek( _from, _count );

	//assert( DalSeekable<unsigned long>::from() == _from );
	//assert(	DalSeekable<unsigned long>::count() == _count );

	//seekable().seek(
	//	DalSeekable<unsigned long>::from() * column -> elements(),
	//	DalSeekable<unsigned long>::count() * column -> elements() );

	DalSeekable<unsigned long>	oldSeek = seekable();

	seekable().seek(
			_from * column -> elements(),
			_count * column -> elements() );

	dataStorage() -> initialise( oldSeek, seekable() );

	ENDPROC
}


const longVec&
HiLowColumnData::size() const
{
	PROC
	RETURN this -> DalChild<Column>::parent() -> size();
}


unsigned long
HiLowColumnData::elements() const
{
	PROC
	RETURN this -> DalChild<Column>::parent() -> elements();
}


template<class T>
static void dumpData(
	const HiLowColumnData *	columnData,
	T			values,
	unsigned long 		from,
	unsigned long 		count )
{
	PROC

	const DalColumn * column = columnData -> DalChild<DalColumn>::parent();

	if( columnData -> mode() == Dal::Read )
	{
		// ***************
		// General Message
		// ***************
		if( errHandler.verbosity(Msg::App) >= Msg::Verbose )
		{
			errstr << "Data for column with qualified name " << column -> qualifiedName() << " (column number is " << column -> columnNumber() << ") from row " << from << " to row " << from + count - 1 << " is read-only, and will not be written back to disk." << Sas::message( Msg::Core, Msg::Verbose );
		} // if

		ALTRETURN;
	} // if

	const DalTable * table = column -> DalChild<DalTable>::parent();
	const DalDataSet * dataSet = ((DalChild<DalDataSet>&)*table).parent();
        //dataSet -> dataSetReaderWriter() -> dumpData( *column, values, from, count );
	const unsigned long e = column -> elements();

        dataSet -> dataSetReaderWriter() -> dumpData( *column, values, from / e, count );
	ENDPROC
}


template<class T>
static void loadData(
	const HiLowColumnData *	columnData,
	T			values,
	unsigned long 		from,
	unsigned long 		count )
{
	PROC
	const DalColumn * column = columnData -> DalChild<DalColumn>::parent();
	const DalTable * table = column -> DalChild<DalTable>::parent();
	const DalDataSet * dataSet = ((DalChild<DalDataSet>&)*table).parent();
        //dataSet -> dataSetReaderWriter() -> loadData( *column, values, from, count );
	const unsigned long e = column -> elements();

        dataSet -> dataSetReaderWriter() -> loadData( *column, values, from / e, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( char ** values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( bool8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( int8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( uint16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( int16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( uint32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( int32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( real32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::dumpData( real64 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::dumpData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( char ** values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( bool8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( int8 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( uint16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( int16 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( uint32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( int32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( real32 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::loadData( real64 * values, unsigned long from, unsigned long count ) const
{
	PROC
	::loadData( this, values, from, count );
	ENDPROC
}


void
HiLowColumnData::copyData( unsigned long from, unsigned long to, unsigned long count ) const
{
	PROC

	const DalColumn * column = this -> DalChild<DalColumn>::parent();
	const DalTable * table = ((DalChild<DalTable>&)*column).parent();
	const DalDataSet * dataSet = ((DalChild<DalDataSet>&)*table).parent();
	const unsigned long n = column -> elements();
        dataSet -> dataSetReaderWriter() -> copyRows( *column, from / n, to / n, count / n );

	ENDPROC
}


int8 *
HiLowColumnData::int8Data() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> int8Data( seekable().from(), seekable().count() );
}


uint16 *
HiLowColumnData::uint16Data() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> uint16Data( seekable().from(), seekable().count() );
}


int16 *
HiLowColumnData::int16Data() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> int16Data( seekable().from(), seekable().count() );
}


uint32 *
HiLowColumnData::uint32Data() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> uint32Data( seekable().from(), seekable().count() );
}


int32 *
HiLowColumnData::int32Data() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> int32Data( seekable().from(), seekable().count() );
}


real32 *
HiLowColumnData::real32Data() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> real32Data( seekable().from(), seekable().count() );
}


real64 *
HiLowColumnData::real64Data() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> real64Data( seekable().from(), seekable().count() );
}


bool8 *
HiLowColumnData::boolData() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> boolData( seekable().from(), seekable().count() );
}


char *
HiLowColumnData::stringData() const
{
	PROC
	((Storage<HiLowColumnData>&)*dataStorage()).parent( this );
	RETURN dataStorage() -> stringData( seekable().from(), seekable().count() );
}
